/***************************************************************************
 *   Copyright (C) 2005-2006 Nicolas Hadacek <hadacek@kde.org>             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef MISC_H
#define MISC_H

#include <ctype.h>
#include "common/global/global.h"
#include <qstringlist.h>
#include <qmap.h>

#include "number.h"

//-----------------------------------------------------------------------------
inline QString repeat(const char *r, uint nb) {
  QString s;
  for (uint i=0; i<nb; i++) s += r;
  return s;
}

inline QString stripEndingWhiteSpaces(const QString &s) {
  int k = s.length()-1;
  for (; k>=0; k--) if ( s[k]!=' ' ) break;
  return s.mid(0, k+1);
}

extern uchar bin2bcd(uchar bin);
extern uchar bcd2bin(uchar bcd);
inline bool XOR(bool b1, bool b2) { return ( (!b1 && b2) || (b1 && !b2) ); }
extern Q_UINT32 getULong(const QByteArray &data, uint offset, uint nbBytes, bool *ok);

inline QString toPrintable(char c) {
  QString s;
  if ( isprint(c) && c!='\\' ) s += c;
  else s += "\\" + toHex(uchar(c), 2);
  return s;
}
inline QString toPrintable(const char *data, uint size) {
  QString s;
  for (uint i=0; i<size; i++) s += toPrintable(data[i]);
  return s;
}
inline QString toPrintable(const QString &s) {
  QByteArray a = toAscii(s);
  return toPrintable(a.data(), a.count()); }

extern QString escapeXml(const QString &s);
extern QString htmlTableRow(const QString &title, const QString &value);

inline void qCrash() {
  char *ptr = 0;
  *ptr = 0;
}

//-----------------------------------------------------------------------------
class Range
{
public:
  Range() : start(0), end(0) {}
  Range(uint s, uint e) : start(s), end(e) {}
  bool isEmpty() const { return end<=start; }
  bool contains(uint v) const { return !isEmpty() && v>=start && v<=end; }

  uint start, end;
};

class RangeVector : public QValueVector<Range>
{
public:
  RangeVector() {}
  bool isEmpty() const {
    for (uint i=0; i<uint(count()); i++) if ( !at(i).isEmpty() ) return false;
    return true;
  }
  bool contains(uint v) const {
    for (uint i=0; i<uint(count()); i++) if ( at(i).contains(v) ) return true;
    return false;
  }
};

//-----------------------------------------------------------------------------
class VersionData
{
public:
  static VersionData fromString(const QString &s);
  static VersionData fromHexString(const QString &s);

public:
  VersionData() : _valid(false) {}
  VersionData(uchar majorNum, uchar midNum, uchar minorNum)
    : _valid(true), _majorNum(majorNum), _midNum(midNum), _minorNum(minorNum) {}
  bool isValid() const { return _valid; }
  void clear() { _valid = false; }
  uchar majorNum() const { return _majorNum; }
  uchar midNum() const { return _midNum; }
  uchar minorNum() const { return _minorNum; }
  QString sub() const { return _sub; }
  QString pretty() const;
  uint toUInt() const;
  bool operator <(const VersionData &vd) const { return toUInt()<vd.toUInt(); }
  bool operator <=(const VersionData &vd) const { return toUInt()<=vd.toUInt(); }
  bool operator >(const VersionData &vd) const { return toUInt()>vd.toUInt(); }
  bool operator >=(const VersionData &vd) const { return toUInt()>=vd.toUInt(); }
  bool operator ==(const VersionData &vd) const { return toUInt()==vd.toUInt(); }
  bool operator !=(const VersionData &vd) const { return toUInt()!=vd.toUInt(); }

private:
  bool    _valid;
  uchar   _majorNum, _midNum, _minorNum;
  QString _sub;
};

#endif
