/*  Copyright (C) 2000 David Faure <faure@kde.org>, Alexander Neundorf <neundorf@kde.org>
    Copyright (C) 2006 Nicolas Hadacek <hadacek@kde.org>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/
#ifndef EDITLISTBOX_H
#define EDITLISTBOX_H

#include <qlayout.h>
#include <qlistbox.h>
#include <klineedit.h>
#include <kcombobox.h>
#include <kpushbutton.h>

#include "common/common/qflags.h"

//----------------------------------------------------------------------------
// QListBox where the selected item can be deselected by
// either cliking on it again or cliking somewhere else
class ListBox : public QListBox
{
Q_OBJECT
public:
  ListBox(QWidget *parent = 0, const char *name = 0);

private slots:
  void clickedSlot(QListBoxItem *item);
};

//----------------------------------------------------------------------------
// modified KEditListBox with:
// * support for duplicated items
// * use KStdGuiItem for buttons
// * support for New, Clear, Reset buttons
// * use a deselectable ListBox
class EditListBox : public QFrame
{
Q_OBJECT
public:
    class CustomEditor {
    public:
        CustomEditor() : m_representationWidget( 0L ), m_lineEdit( 0L ) {}
        CustomEditor( QWidget *repWidget, KLineEdit *edit ) : m_representationWidget( repWidget ), m_lineEdit( edit ) {}
        CustomEditor( KComboBox *combo );

        void setRepresentationWidget( QWidget *repWidget ) { m_representationWidget = repWidget; }
        void setLineEdit( KLineEdit *edit ) { m_lineEdit = edit; }

        virtual QWidget   *representationWidget() const { return m_representationWidget; }
        virtual KLineEdit *lineEdit() const { return m_lineEdit; }

    protected:
        QWidget *m_representationWidget;
        KLineEdit *m_lineEdit;
    };

   public:
      enum Mode { DuplicatesDisallowed, DuplicatesAllowed, DuplicatesCheckedAtEntering };
      enum Button { Add = 1, Remove = 2, UpDown = 4, RemoveAll = 8, Reset = 16 };
      Q_DECLARE_FLAGS(Buttons, Button)

      EditListBox(QWidget *parent = 0, const char *name = 0,
	          Mode mode = DuplicatesDisallowed, Buttons buttons = Buttons(Add|Remove|RemoveAll|UpDown) );
      EditListBox(const CustomEditor &customEditor, QWidget *parent = 0, const char *name = 0,
                  Mode mode = DuplicatesDisallowed, Buttons buttons = Buttons(Add|Remove|RemoveAll|UpDown) );
      int count() const { return int(m_listBox->count()); }
      void setItems(const QStringList& items);
      void insertStringList(const QStringList& list, int index=-1);
      void insertItem(const QString& text, int index=-1) {m_listBox->insertItem(text,index);}
      QString text(int index) const { return m_listBox->text(index); }
      QStringList items() const;
      Buttons buttons() const { return _buttons; }
      void setButtons(Buttons buttons);

   signals:
      void reset();
      void changed();
      void added( const QString & text );
      void removed( const QString & text );

   public slots:
      void clear();

   protected slots:
      void moveItemUp();
      void moveItemDown();
      void clearEdit();
      void addItem();
      void removeItem();
      void enableMoveButtons(int index);
      void typedSomething(const QString& text);

   private:
      Mode         _mode;
      Buttons      _buttons;
      QVBoxLayout *_buttonsLayout;
      ListBox     *m_listBox;
      KLineEdit   *m_lineEdit;
      KPushButton *_addButton, *_removeButton, *_moveUpButton, *_moveDownButton,
                  *_removeAllButton, *_resetButton;

      void init(QWidget *representationWidget);
      void updateButtons();
};
Q_DECLARE_OPERATORS_FOR_FLAGS(EditListBox::Buttons)

#endif
