/***************************************************************************
 *   Copyright (C) 2006 Nicolas Hadacek <hadacek@kde.org>                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef MISC_GUI_H
#define MISC_GUI_H

#include <qlayout.h>
#include <qsplitter.h>
#include <qvaluevector.h>

#include <klocale.h>
#include <kpushbutton.h>
#include <ktabwidget.h>
#include <ktabbar.h>
class KAction;

#include "common/global/generic_config.h"
#include "common/global/log.h"

//-----------------------------------------------------------------------------
class BusyCursor
{
public:
  BusyCursor() { start(); }
  ~BusyCursor() { stop(); }
  static void start();
  static void stop();
  static void pause();
  static void restore();

private:
  static bool _overridePaused;
};

//-----------------------------------------------------------------------------
namespace MessageBox
{
extern void information(const QString &text, Log::ShowMode show);
extern void detailedSorry(const QString &text, const QString &details, Log::ShowMode show);
inline void sorry(const QString &text, Log::ShowMode show) { detailedSorry(text, QString::null, show); }
extern bool askContinue(const QString &text, const QString &caption = i18n("Warning"),
                        const KGuiItem &buttonContinue = KStdGuiItem::cont());
extern void text(const QString &text, Log::ShowMode show);
}

//----------------------------------------------------------------------------
class PopupButton : public KPushButton
{
Q_OBJECT
public:
  PopupButton(QWidget *parent = 0, const char *name = 0);
  PopupButton(const QString &text, QWidget *parent = 0, const char *name = 0);
  void appendAction(KAction *action);
  void appendAction(const QString &label, const QString &icon = QString::null,
                   QObject *receiver = 0, const char *slot = 0);
  void appendItem(const QString &label, uint id) { appendItem(label, QPixmap(), id); }
  void appendItem(const QString &label, const QString &icon, uint id);
  void appendItem(const QString &label, const QPixmap &icon, uint id);
  void appendSeparator() { _separator = true; }

signals:
  void activated(int id);

private:
  bool _separator;

  void init();
};

//-----------------------------------------------------------------------------
class Splitter : public QSplitter
{
Q_OBJECT
public:
  Splitter(const QValueList<int> &defaultSizes, Orientation orientation,
           QWidget *parent, const char *name);
  virtual ~Splitter();

private slots:
  void updateSizes();

private:
  QValueList<int> _defaultSizes;
};

//-----------------------------------------------------------------------------
class GuiConfig : public GenericConfig
{
public:
  GuiConfig() : GenericConfig("gui") {}
};

//-----------------------------------------------------------------------------
class SeparatorWidget : public QFrame
{
Q_OBJECT
public:
  SeparatorWidget(QWidget *parent) : QFrame(parent, "separator") {
    setFrameStyle(QFrame::Panel | QFrame::Sunken);
    setMargin(2);
    setFixedHeight(2*2);
  }
};

//-----------------------------------------------------------------------------
class TabBar : public KTabBar
{
Q_OBJECT
public:
  TabBar(QWidget *parent = 0, const char *name = 0);

protected:
  virtual void wheelEvent(QWheelEvent *e);

private:
  bool _ignoreWheelEvent;

  friend class TabWidget;
};

class TabWidget : public KTabWidget
{
Q_OBJECT
public:
  TabWidget(QWidget *parent = 0, const char *name = 0);
  void setIgnoreWheelEvent(bool ignore);

protected:
  virtual void wheelEvent(QWheelEvent *e);
  void setTabBar(TabBar *tabbar);
};

#endif
