/***************************************************************************
 *   Copyright (C) 2006 Nicolas Hadacek <hadacek@kde.org>                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "generic_memory.h"

#include <qfile.h>
#include <qtextstream.h>

bool Device::Memory::load(QFile &file, QString &error, WarningTypes &warningTypes, QStringList &warnings)
{
  HexBuffer hb;
  if ( !hb.load(file, error) ) return false;
  warningTypes = fromHexBuffer(hb, warnings);
  return true;
}

Device::Memory::WarningTypes Device::Memory::fromHexBuffer(const HexBuffer &hb, QStringList &warnings)
{
  clear();
  WarningTypes result = NoWarning;
  warnings.clear();
  QMap<uint, bool> inRange;
  fromHexBuffer(hb, result, warnings, inRange);

   // check that all values in FragBuffer are within memory ranges
  HexBuffer::const_iterator it = hb.begin();
  for (; it!=hb.end(); ++it) {
    if ( (*it).second==Device::INVALID || inRange[(*it).first] ) continue;
    if ( !(result & ValueOutsideRange) ) {
      result |= ValueOutsideRange;
      warnings += i18n("At least one value (at address %1) is defined outside memory ranges.")
                 .arg(toHexLabel((*it).first, 8));
    }
    break;
  }

  return result;
}

bool Device::Memory::save(QFile &file, HexBuffer::Format format) const
{
  QTextStream stream(&file);
  savePartial(stream, format);
  HexBuffer::saveEnd(stream);
  file.close();
  return ( file.status()==int(IO_Ok) );
}
