/***************************************************************************
 *   Copyright (C) 2005 Nicolas Hadacek <hadacek@kde.org>                  *
 *             (C) 2003 by Alain Gibaud                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef HEX_BUFFER_H
#define HEX_BUFFER_H

#include <map>
#include <qstring.h>
class QTextStream;
class QFile;

class HexBuffer : public std::map<uint, uint>
{
public:
  HexBuffer() {}

  void set(uint offset, uint value);
  uint operator[](uint offset) const;
  uint *operator+(uint offset); //emulates a pointer+integer operation
                                // insures that a never used word is read as INVALID

  enum Format { /// Differents flavors of Intel HEX file formats
    IHX8M = 0,  ///< 8 bits "swapped" format
    //    IHX8S,
    IHX16,      ///< 16 bits format
    IHX32,      ///< 8 bit format with 32 bits addresses
    Nb_Formats
  };
  static const char * const FORMATS[Nb_Formats];

  void savePartial(QTextStream &s, Format format) const;
  static void saveEnd(QTextStream &s);
  bool load(QFile &file, QString &error);

private:
  enum { HEXBLKSIZE = 8 }; // line size in HEX files
  static bool fetchNextBlock(const_iterator& start, const const_iterator &end, int *len);
  static void writeHexBlock(QTextStream &stream, int reclen, const_iterator& data, Format format);
  enum ReadError { NoError, CRCError, UnrecognizedError };
  ReadError readHex(QFile &file);
};

#endif
