/***************************************************************************
 *   Copyright (C) 2005 Nicolas Hadacek <hadacek@kde.org>                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "cli_log.h"

#include "common/global/global.h"

Log::ConsoleView *Log::ConsoleView::_self = 0;

void Log::ConsoleView::init(bool interactive)
{
  Q_ASSERT(_self==0);
  _self = new ConsoleView;
  _self->_interactive = interactive;
}

void Log::ConsoleView::doLog(Type type, const QString &text, Action)
{
  QString s = text + "\n";
  switch (type) {
    case Error:
    case SoftError: s = "Error: " + s; break;
    case Warning: s = "Warning: " + s; break;
    default: break;
  }
#if QT_VERSION<0x040000
  if ( type==Error || type==SoftError ) fprintf(stderr, "%s", s.latin1());
  else fprintf(stdout, "%s", s.latin1());
#else
  QByteArray ba = s.toLatin1();
  if ( type==Error || type==SoftError ) fprintf(stderr, "%s", ba.constData());
  else fprintf(stdout, "%s", ba.constData());
#endif

}

void Log::ConsoleView::doLog(DebugLevel, const QString &text, Action)
{
  QString s = text + "\n";
#if QT_VERSION<0x040000
  fprintf(stdout, "%s", s.latin1());
#else
  QByteArray ba = s.toLatin1();
  fprintf(stdout, "%s", ba.constData());
#endif
}

void Log::ConsoleView::appendToLastLine(const QString &text)
{
#if QT_VERSION<0x040000
  fprintf(stdout, "%s", text.latin1());
#else
  QByteArray ba = text.toLatin1();
  fprintf(stdout, "%s", ba.constData());
#endif
}

void Log::ConsoleView::sorry(const QString &message, const QString &details)
{
  if ( details.isEmpty() ) log(Error, message, Immediate);
  else log(Error, message + ": " + details, Immediate);
}

bool Log::ConsoleView::askContinue(const QString &message)
{
  log(Warning, message + " " + i18n("no"), Immediate);
  if ( !_interactive ) return false; // always fail
  // #### TODO
  return false;
}
