/***************************************************************************
 *   Copyright (C) 2005 Nicolas Hadacek <hadacek@kde.org>                  *
 *   Copyright (C) 2003 Alain Gibaud <alain.gibaud@free.fr>                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef HEX_EDITOR_H
#define HEX_EDITOR_H

#include "device_editor.h"
#include "devices/base/hex_buffer.h"
#include "devices/base/generic_memory.h"

class HexEditor : public DeviceEditor
{
Q_OBJECT
public:
  HexEditor(const QString &name, QWidget *parent);
  HexEditor(const PURL::Url &url, QWidget *parent);
  virtual ~HexEditor() { clear(); }
  virtual PURL::FileType fileType() const { return PURL::Hex; }
  virtual bool state(ValueType type) const;
  virtual void setState(ValueType type, bool on);
  virtual bool save(const PURL::Url &url) const;
  Device::Memory *memory() { return _memory; }
  const Device::Memory *memory() const { return _memory; }
  const Device::Memory *originalMemory() const { return _originalMemory; }
  void memoryRead();

public slots:
  virtual void statusChanged();

private slots:
  void slotModified();
  bool simpleLoad();

private:
  Device::Memory *_originalMemory, *_memory;
  HexBuffer       _hexBuffer;
  bool            _modified, _readWrite, _dirty;

  virtual void dropEvent(QDropEvent *e) { emit dropEventPass(e); }
  bool save(const PURL::Url &url, const QString &fileErrorString) const;
  bool verifyDeviceType();
  virtual bool load();
  virtual QWidget *createView(const Device::Data *data, QWidget *parent);
  void display();
  void init();
  void clear();
};

#endif
