/***************************************************************************
 *   Copyright (C) 2005 Nicolas Hadacek <hadacek@kde.org>                  *
 *   Copyright (C) 2003-2004 Alain Gibaud <alain.gibaud@free.fr>           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef TEXT_EDITOR_H
#define TEXT_EDITOR_H

#include <qsplitter.h>
#include <qstringlist.h>
class QSplitter;

#include <kate/view.h>
#include <kate/document.h>

#include "common/global/pfile.h"
#include "editor.h"
#include "progs/manager/breakpoint.h"

//-----------------------------------------------------------------------------
class TextEditor : public Editor
{
Q_OBJECT
public:
  TextEditor(Capabilities capabilities, const PURL::Url &url, QWidget *parent, const char *name = 0);
  virtual PURL::FileType fileType() const { return url().fileType(); }
  virtual bool state(ValueType type) const;
  virtual void setState(ValueType type, bool on);
  virtual void setCursor(uint line, uint column) { _view->setCursorPosition(line, column); }
  virtual void configureEditor() { _document->configDialog(); }
  virtual void setFocus() { _view->setFocus(); }
  uint cursorLine() const;

  static QPixmap pixmap(Breakpoint::MarkType type);
  void setMark(uint line, Breakpoint::MarkType type);
  void clearMarks();

public slots:
  void addView();
  void removeCurrentView();
  void selectAll() { _document->selectAll(); }
  void deselect() { _document->clearSelection(); }
  void undo() { _document->undo(); }
  void redo() { _document->redo(); }
  void cut() { _view->cut(); }
  void copy() { _view->copy(); }
  void paste() { _view->paste(); }
  void search() { _view->find(); }
  void replace() { _view->replace(); }
  void gotoLine() { _view->gotoLine(); }
  void statusChanged();
  void uncomment() { _view->uncomment(); }
  void comment() { _view->comment(); }
  void findNext() { _view->findAgain(); }
  void findPrevious() { _view->findPrev(); }
  void printDialog() { _document->printDialog(); }
  void gotFocus(Kate::View *);
  void highlightChanged();

protected:
  enum EolType { Dos = 1, Unix = 0, Mac = 2 };
  Kate::Document *_document;
  Kate::View     *_view;

private slots:
  void addToDebugManager();

private:
  QSplitter *_split;
  bool       _oldModified;
  struct MarkTypeData {
    int value;
    const char *pixmap;
  };
  static const MarkTypeData MARK_TYPE_DATA[Breakpoint::Nb_MarkTypes];

  virtual bool save(const PURL::Url &url) const { return _document->saveAs(url.kurl()); }
  virtual bool load();
  uint highlightMode(const QString &name) const;
};

//-----------------------------------------------------------------------------
class SimpleTextEditor : public TextEditor
{
Q_OBJECT
public:
  SimpleTextEditor(Capabilities capabilities, PURL::FileType type, QWidget *parent, const char *name = 0);
  SimpleTextEditor(Capabilities capabilities, const PURL::Url &url, QWidget *parent, const char *name = 0);
  void setFileType(PURL::FileType type) { _type = type; }
  virtual PURL::FileType fileType() const { return _type; }
  bool setText(const QString &text);

protected:
  virtual bool load() { return false; }

private:
  PURL::FileType _type;
  PURL::TempFile _file;
};

#endif
