/***************************************************************************
 *   Copyright (C) 2005-2006 Nicolas Hadacek <hadacek@kde.org>             *
 *   Copyright (C) 2003-2004 Alain Gibaud <alain.gibaud@free.fr>           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef PIKMAIN_H
#define PIKMAIN_H

class QLabel;
class QProgressBar;
#include <kdockwidget.h>
#include <kaction.h>

#include "common/gui/misc_gui.h"
#include "config_center.h"
#include "tools/base/compile.h"
namespace Programmer { class Group; }
namespace Tool { class Group; }
class ConfigGenerator;
class HexEditor;
namespace Process { class Base; }

//----------------------------------------------------------------------------
class ProgrammerStatusWidget : public PopupButton
{
Q_OBJECT
public:
  ProgrammerStatusWidget(QWidget *parent);

signals:
  void configure();
  void selected(const Programmer::Group &group);

private slots:
  void activatedSlot(int i);
};

//----------------------------------------------------------------------------
class ToolStatusWidget : public PopupButton
{
Q_OBJECT
public:
  ToolStatusWidget(QWidget *parent);

signals:
  void configureToolchain();
  void configure();
  void selected(const Tool::Group &group);

private slots:
  void activatedSlot(int i);
};

//----------------------------------------------------------------------------
class ViewMenuAction : public KAction
{
Q_OBJECT
public:
  ViewMenuAction(QWidget *widget, bool isVisible, const QString &name);

signals:
  void activated(QWidget *);

private slots:
  virtual void slotActivated();

private:
  QWidget *_widget;
};

//----------------------------------------------------------------------------
class MainWindow : public KDockMainWindow
{
Q_OBJECT
public:
  MainWindow();
  virtual ~MainWindow();
  void setDevice(const QString &device);

public slots:
  void newSourceFile();
  void newProject();
  void openProject();
  void buildProject();
  void cleanBuild();
  void configure(ConfigCenter::Type showType = ConfigCenter::General);
  void configureProject();
  void showDeviceInfo();
  void updateGUI();

private slots:
  void newHexFile();
  bool openFile();
  void openRecentFile(const KURL &url);

  void compileFile();
  void compileSuccess();
  void compileFailure();
  void updateFile(const Compile::FileData &data);
  void stopBuild();

  void program();
  void verify();
  void read();
  void setTotalSteps(uint total);
  void addToTotalSteps(uint add);
  void addProgress(uint steps);

  void applyToolbarSettings();
  void configureToolbar();
  void configureEditor();
  void configureToolchains();
  void configureProgrammer() { configure(ConfigCenter::ProgSelect); }
  void selectProgrammer(const Programmer::Group &group);
  void selectTool(const Tool::Group &group);

  void fillToolViewsMenu();
  void showAllToolViews();
  void toggleToolView(QWidget *widget);

  void runPikloops();
  void pikloopDone();
  void configGenerator();

  void openRecentProject(const KURL &url);
  void closeProject();

  void initialLoading();

signals:
  void stateChanged();

private:
  Log::Widget            *_programLog;
  QLabel                 *_actionStatus, *_debugStatus, *_editorStatus;
  ProgrammerStatusWidget *_programmerStatus;
  ToolStatusWidget       *_toolStatus;
  QProgressBar           *_actionProgress;
  ConfigGenerator        *_configGenerator;
  ::Process::Base        *_pikloop;

  KActionMenu          *_toolViewsMenu;
  QValueList<QWidget *> _views;

  void saveSettings();
  HexEditor *getHexEditor();
  virtual bool queryClose();
  void stopOperations();
  void cleanBuild(bool singleFile);
  virtual void keyPressEvent(QKeyEvent *e);
  void readDockConfig();
};

#endif
