/***************************************************************************
 *   Copyright (C) 2005 Nicolas Hadacek <hadacek@kde.org>                  *
 *   Copyright (C) 2003-2005 Alain Gibaud <alain.gibaud@free.fr>           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "direct_prog_config.h"

#include "common/global/global.h"

const Direct::StandardHardwareData Direct::Config::STANDARD_HARDWARE_DATA[] = {
  { "Tait classic",           I18N_NOOP("Tait classic"),           0,
    { Port::ParallelType, { {  -5, -4,  3,  2, 10, 25 },  0 } } },
  { "Tait 7405/7406",         I18N_NOOP("Tait 7405/7406"),         0,
    { Port::ParallelType, { {   5,  4, -3, -2,-10, 25 },  0 } } },
  { "P16PRO40 classic",       I18N_NOOP("P16PRO40 classic"),       0,
    { Port::ParallelType, { {   5,  4, -3, -2,-10, 25 },  0 } } },
  { "P16PRO40 7407",          I18N_NOOP("P16PRO40 7407"),          0,
    { Port::ParallelType, { {  -5, -4,  3,  2, 10, 25 },  0 } } },
  { "P16PRO40-VPP40 classic", I18N_NOOP("P16PRO40-VPP40 classic"), 0,
    { Port::ParallelType, { {   6,  4, -3, -2,-10, 25 },  0 } } },
  { "P16PRO40-VPP40 7407",    I18N_NOOP("P16PRO40-VPP40 7407"),    0,
    { Port::ParallelType, { {  -6, -4,  3,  2, 10, 25 },  0 } } },
  { "EPIC+",                  I18N_NOOP("EPIC+"),                  I18N_NOOP("You must disconnect 7407 pin 2"),
    { Port::ParallelType, { {  -5, -4,  3,  2, 10, 25 },  0  } } },
  { "JDM classic",            I18N_NOOP("JDM classic"),            0,
    { Port::SerialType,   { {   3,  5,  7,  4,  8,  5 },  0  } } },
  { "JDM classic (delay 10)", I18N_NOOP("JDM classic (delay 10)"), 0,
    { Port::SerialType,   { {   3,  5,  7,  4,  8,  5 }, 10  } } },
  { "JDM classic (delay 20)", I18N_NOOP("JDM classic (delay 20)"), 0,
    { Port::SerialType,   { {   3,  5,  7,  4,  8,  5 }, 20  } } },
  { "PIC Elmer",              I18N_NOOP("PIC Elmer"),              0,
    { Port::SerialType,   { {  -3,  5, -7, -4, -8,  5 },  0 } } },
  { "Velleman K8048",         I18N_NOOP("Velleman K8048"),         0,
    { Port::SerialType,   { {  -3,  5, -7, -4, -8,  5 },  0 } } },

  //Added by Mirko Panciri 10/03/2004...
  //Visit http://www.pic-tools.com
  //I have tested only the "Asix Piccolo" version...
  //I think the lines is the same of "Asix Piccolo Grande"...
  //Who have "Asix Piccolo Grande" contact panciom@libero.it for info / result of test!!!
  { "Asix Piccolo",           I18N_NOOP("Asix Piccolo"),           0,
    { Port::ParallelType, { {  -6, -7, -5, -3,-10, -2 },  0 } } },
  { "Asix Piccolo Grande",    I18N_NOOP("Asix Piccolo Grande"),    0,
    { Port::ParallelType, { {  -6, -7, -5, -3,-10, -2 },  0 } } },

  { "Propic2 Vpp-1",          I18N_NOOP("Propic2 Vpp-1"),          0,
    { Port::ParallelType, { {  -5, -4,  3,  2, 10, 25 },  0 } } },
  { "Propic2 Vpp-2",          I18N_NOOP("Propic2 Vpp-2"),          0,
    { Port::ParallelType, { {  -6, -4,  3,  2, 10, 25 },  0 } } },
  { "Propic2 Vpp-3",          I18N_NOOP("Propic2 Vpp-3"),          0,
    { Port::ParallelType, { {  -7, -4,  3,  2, 10, 25 },  0 } } },
  { "Myke's EL Cheapo",       I18N_NOOP("Myke's EL Cheapo"),       0,
    { Port::ParallelType, { {  16, 25, -1,-17, 13, 25 },  0 } } },
  { "EL Cheapo classic",      I18N_NOOP("EL Cheapo classic"),      I18N_NOOP("not tested"),
    { Port::ParallelType, { {  16, 25,  1, 17,-13, 25 },  0 } } },
  { "Monty-Robot programmer", I18N_NOOP("Monty-Robot programmer"), 0,
    { Port::ParallelType, { {  -5,  4,  2,  3, 10, 25 },  0 } } },
  { "HOODMICRO",              I18N_NOOP("HOODMICRO"),              I18N_NOOP("htpp://k9spud.com/hoodmicro"),
    { Port::SerialType,   { {   4,  5,  7,  3,  8,  5 },  0 } } },

  { 0, 0, 0,
    { Port::SerialType,   { {   0,  0,  0,  0,  0,  0 },  0 } } }
};

void Direct::Config::writeCurrentHardware(Port::Type type, const QString &name)
{
  writeEntry(QString("current_hardware_") + Port::DATA[type].key, name);
}

QString Direct::Config::currentHardware(Port::Type type)
{
  QStringList names = hardwareNames(type);
  return readEntry(QString("current_hardware_") + Port::DATA[type].key, names[0]);
}

const Direct::StandardHardwareData *Direct::Config::standardHardwareData(const QString &name)
{
  for (uint i=0; STANDARD_HARDWARE_DATA[i].name; i++)
    if ( STANDARD_HARDWARE_DATA[i].name==name ) return &STANDARD_HARDWARE_DATA[i];
  return 0;
}

QStringList Direct::Config::hardwareNames(Port::Type type)
{
  QStringList names;
  for (uint i=0; STANDARD_HARDWARE_DATA[i].name; i++) {
    if ( type!=Port::Nb_Types && STANDARD_HARDWARE_DATA[i].hdata.portType!=type ) continue;
    names += STANDARD_HARDWARE_DATA[i].name;
  }
  QStringList customNames = readListEntry("custom_hardware_names", QStringList());
  for (uint i=0; i<uint(customNames.count()); i++) {
    if ( type!=Port::Nb_Types && hardware(customNames[i]).portType!=type ) continue;
    names += customNames[i];
  }
  return names;
}

Direct::HardwareData Direct::Config::hardware(const QString &name)
{
  for (uint i=0; STANDARD_HARDWARE_DATA[i].name; i++)
    if ( STANDARD_HARDWARE_DATA[i].name==name ) return STANDARD_HARDWARE_DATA[i].hdata;

  GenericConfig config("direct_programmer_custom_hardware_" + name);
  HardwareData hdata = { Port::Nb_Types, {{}, 0} };
  QString s = config.readEntry("port_type");
  for (uint i=0; i<Port::Nb_Types; i++ )
    if ( s==Port::DATA[i].key ) hdata.portType = Port::Type(i);
  for (uint i=0; i<Nb_PinTypes; i++) hdata.data.pins[i] = config.readIntEntry(PIN_DATA[i].key);
  hdata.data.clockDelay = config.readIntEntry("clkdelay");
  return hdata;
}

void Direct::Config::writeCustomHardware(const QString& name, const HardwareData &hdata)
{
  Q_ASSERT( !isStandardHardware(name) );
  QStringList customNames = readListEntry("custom_hardware_names", QStringList());
  if ( !customNames.contains(name) ) {
    customNames += name;
    writeEntry("custom_hardware_names", customNames);
  }
  GenericConfig config("direct_programmer_custom_hardware_" + name);
  config.writeEntry("port_type", Port::DATA[hdata.portType].key);
  const Hardware::HData &data = hdata.data;
  for (uint i=0; i<Nb_PinTypes; i++) config.writeEntry(PIN_DATA[i].key, data.pins[i]);
  config.writeEntry("clkdelay", data.clockDelay);
}

void Direct::Config::deleteCustomHardware(const QString &name)
{
  Q_ASSERT( !isStandardHardware(name) );
  QStringList customNames = readListEntry("custom_hardware_names", QStringList());
  customNames.remove(name);
  writeEntry("custom_hardware_names", customNames);
  GenericConfig::deleteGroup("direct_programmer_custom_hardware_" + name);
}
