/***************************************************************************
 *   Copyright (C) 2005 Nicolas Hadacek <hadacek@kde.org>                  *
 *   Copyright (C) 2003-2004 Alain Gibaud <alain.gibaud@free.fr>           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef DIRECT_CONFIG_WIDGET
#define DIRECT_CONFIG_WIDGET

#include <qcombobox.h>
#include <qcheckbox.h>
#include <qpushbutton.h>
#include <qgroupbox.h>
#include <qtimer.h>
#include <qlineedit.h>
#include <knuminput.h>

#include "progs/gui/prog_group_ui.h"
#include "progs/gui/prog_config_widget.h"
#include "progs/direct/base/direct.h"

namespace Direct
{
//----------------------------------------------------------------------------
class GroupUI : public ::Programmer::GroupUI
{
public:
  virtual ::Programmer::ConfigWidget *createConfigWidget(QWidget *parent) const;
  virtual bool hasAdvancedDialog() const { return false; }
  virtual ::Programmer::AdvancedDialog *createAdvancedDialog(::Programmer::Base &, QWidget *) const { return 0; }
};

//-----------------------------------------------------------------------------
class HardwareConfigWidget : public QFrame
{
Q_OBJECT
public:
  HardwareConfigWidget(const ::Programmer::Group &gorup, QWidget *parent, bool edit);
  virtual ~HardwareConfigWidget();

  bool set(const Port::Description &pd, const Hardware::HData &data);
  Hardware::HData data() const;
  Port::Description portDescription() const { return _hardware->portDescription(); }

private slots:
  void slotTestPin();
  void slotPinChanged();
  void updateDataIn();
  void sendBits();
  void slotSendBits();

private:
  bool              _edit, _connected;
  Hardware         *_hardware;
  ::Programmer::Base *_base;

  QComboBox    *_combos[Nb_PinTypes];
  QCheckBox    *_invcbs[Nb_PinTypes];
  QCheckBox    *_testcbs[Nb_PinTypes];
  QLabel       *_testLabels[Nb_PinTypes];
  KIntNumInput *_delay;
  QPushButton  *_sendBitsButton;
  QTimer       *_timerSendBits, *_timerPollDataOut;

  void sendBits(uint d, int nbb);
  void updateTestPin(PinType ptype);
  void updateTestStatus(PinType ptype, bool on);
  uint pin(PinType ptype) const;
  void updatePin(PinType ptype);
};

//-----------------------------------------------------------------------------
class HardwareConfigDialog : public KDialogBase
{
Q_OBJECT
public:
  HardwareConfigDialog(const ::Programmer::Group &group, QWidget *parent, const QString &name,
                       const Port::Description &pd, const Hardware::HData &data);
  QString savedName() const { return _savedName; }

private slots:
  virtual void slotOk();
  virtual void slotCancel();

private:
  QString               _savedName;
  Hardware::HData       _oldData;
  HardwareConfigWidget *_hc;
  QLineEdit            *_name;
};

//-----------------------------------------------------------------------------
class ConfigWidget : public ::Programmer::ConfigWidget
{
Q_OBJECT
public:
  ConfigWidget(const ::Programmer::Group &group, QWidget *parent);
  virtual void saveConfig();
  virtual bool setPort(const Port::Description &pd);

private slots:
  void editClicked();
  void deleteClicked();
  void configChanged(int i);

private:
  QStringList    _names;
  HardwareConfigWidget *_hc;
  QPushButton    *_deleteButton;
  QComboBox      *_configCombo;
  QLabel         *_comment;

  void updateList(Port::Type type);
  bool set(const Port::Description &pd, uint i);
};

} // namespace

#endif
