/***************************************************************************
 *   Copyright (C) 2006 Nicolas Hadacek <hadacek@kde.org>                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef BOOST_H
#define BOOST_H

#include "tools/base/tool.h"
#include "common/global/pfile.h"
#include "devices/pic/base/pic.h"

namespace Boost
{
//----------------------------------------------------------------------------
class Linker : public Tool::Base
{
private:
  virtual QString baseExecutable(bool) const { return "boostlink.pic.exe"; }
  virtual bool checkExecutableResult(bool withWine, QStringList &lines) const;
  virtual PURL::Directory checkExecutableWorkingDirectory() const;
};

class Linker16 : public Linker
{
private:
  virtual QStringList checkExecutableOptions() const;
};

class Linker18 : public Linker
{
private:
  virtual QStringList checkExecutableOptions() const;
};

//----------------------------------------------------------------------------
class Compiler : public Tool::Base
{
private:
  virtual QStringList checkExecutableOptions() const;
  virtual PURL::Directory checkExecutableWorkingDirectory() const;
};

//----------------------------------------------------------------------------
class Group : public Tool::Group
{
public:
  Group(Pic::Architecture architecture, const QString &extension, const QString &text);
  PURL::Url checkExecutableUrl() const { return _checkExecTmp.url(); }
  virtual Tool::Category checkDevicesCategory() const { return Tool::Nb_Categories; }
  virtual Tool::ExecutableType preferedExecutableType() const { return Tool::Windows; }
  virtual bool hasDirectory(Compile::DirectoryType type) const { return type==Compile::LibraryDirectory; }
  virtual PURL::Directory autodetectDirectory(Compile::DirectoryType type, const PURL::Directory &execDir, bool withWine) const;
  virtual bool needs(Tool::ActionType, Tool::Category category) const { return ( category==Tool::Linker ); }
  virtual QString generateDeviceConfig(PURL::ToolType type, const Device::Memory &memory, bool &ok) const;

protected:
  Pic::Architecture _architecture;

private:
  PURL::TempFile _checkExecTmp;

  virtual QValueList<const Device::Data *> getSupportedDevices(const QString &s) const;
  virtual Compile::Process *processFactory(Tool::Category category, const ::Project *project) const;
  virtual Compile::Config *configFactory(::Project *project) const;
  virtual ToolConfigWidget *configWidgetFactory(Tool::Category category, ::Project *project) const;
};

} // namespace

#endif
