/***************************************************************************
 *   Copyright (C) 2006 Nicolas Hadacek <hadacek@kde.org>                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "ccsc.h"

#include <qregexp.h>

#include "ccsc_compile.h"
#include "ccsc_config.h"
#include "devices/pic/pic/pic_memory.h"
#include "devices/list/device_list.h"
#include "devices/base/device_group.h"

//----------------------------------------------------------------------------
QStringList CCSC::Base::checkExecutableOptions() const
{
  QStringList args;
  args += "+STDERR";
  args += "+EA";
  args += static_cast<const Group *>(_group)->checkExecutableUrl().filename();
  return args;
}

PURL::Directory CCSC::Base::checkExecutableWorkingDirectory() const
{
  return static_cast<const Group *>(_group)->checkExecutableUrl().directory();
}

bool CCSC::Base::checkExecutableResult(bool, QStringList &lines) const
{
  return ( lines.count()>0 && lines[0].startsWith("*** Error ") ); // #### weak test...
}

//----------------------------------------------------------------------------
CCSC::Group::Group()
  : _checkExecTmp(0, ".c")
{
  if ( _checkExecTmp.openForWrite() ) _checkExecTmp.appendText("void main(void) {}\n");
  _checkExecTmp.close();
}

QValueList<const Device::Data *> CCSC::Group::getSupportedDevices(const QString &) const
{
  QValueList<const Device::Data *> list;
  QValueVector<QString> devices = Device::lister().group("pic")->supportedDevices();
  for (uint i=0; i<devices.count(); i++) {
    const Device::Data *data = Device::lister().data(devices[i]);
    if ( static_cast<const Pic::Data *>(data)->architecture()==Pic::Arch_30X ) continue;
    list.append(data);
  }
  return list;
}

Compile::Process *CCSC::Group::processFactory(Tool::Category category, const Project *) const
{
  switch (category) {
    case Tool::Compiler: return new CCSC::CompileFile;
    default: break;
  }
  Q_ASSERT(false);
  return 0;
}

ToolConfigWidget *CCSC::Group::configWidgetFactory(Tool::Category category, Project *project) const
{
  if ( category==Tool::Compiler ) return new CCSC::ConfigWidget(project);
  return 0;
}

Compile::Config *CCSC::Group::configFactory(::Project *project) const
{
  return new Config(project);
}

PURL::FileType CCSC::Group::sourceFileType(PURL::ToolType type) const
{
  if ( type==PURL::Compiler ) return PURL::CSource;
  return PURL::Nb_FileTypes;
}

QString CCSC::Group::generateDeviceConfig(PURL::ToolType, const Device::Memory &memory, bool &ok) const
{
  const Pic::Memory &pmemory = static_cast<const Pic::Memory &>(memory);
  const Pic::Data &data = pmemory.device();
  const Pic::Config &config = data.config();
  ok = true;
  // #### TODO
  return QString::null;
}

QString CCSC::Group::downloadText() const
{
  return i18n("<a href=\"http://www.ccsinfo.com/content.php?page=compilers\">CCS Compiler</a> is a C compiler distributed by CCS.");
}

Tool::Base *CCSC::Group::baseFactory(Tool::Category category) const
{
  if ( category==Tool::Compiler ) return new CCSC::Base;
  return 0;
}
