/***************************************************************************
 *   Copyright (C) 2006 Nicolas Hadacek <hadacek@kde.org>                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef CCSC_H
#define CCSC_H

#include "tools/base/tool.h"
#include "common/global/pfile.h"

namespace CCSC
{
//----------------------------------------------------------------------------
class Base : public Tool::Base
{
public:
  virtual QString baseExecutable(bool) const { return "ccsc"; }

private:
  virtual QStringList checkExecutableOptions() const;
  virtual bool checkExecutableResult(bool withWine, QStringList &lines) const;
  virtual PURL::Directory checkExecutableWorkingDirectory() const;
};

//----------------------------------------------------------------------------
class Group : public Tool::Group
{
public:
  Group();
  PURL::Url checkExecutableUrl() const { return _checkExecTmp.url(); }
  virtual QString name() const { return "ccsc"; }
  virtual QString label() const { return i18n("CCS Compiler"); }
  virtual QString downloadText() const;
  virtual Tool::Category checkDevicesCategory() const { return Tool::Nb_Categories; }
  virtual Tool::ExecutableType preferedExecutableType() const { return Tool::Windows; }
  virtual bool needs(Tool::ActionType, Tool::Category category) const { return ( category==Tool::Compiler ); }
  virtual PURL::FileType sourceFileType(PURL::ToolType type) const;
  virtual QString generateDeviceConfig(PURL::ToolType type, const Device::Memory &memory, bool &ok) const;

private:
  PURL::TempFile _checkExecTmp;

  virtual QValueList<const Device::Data *> getSupportedDevices(const QString &s) const;
  virtual Compile::Process *processFactory(Tool::Category category, const ::Project *project) const;
  virtual Compile::Config *configFactory(::Project *project) const;
  virtual ToolConfigWidget *configWidgetFactory(Tool::Category category, ::Project *project) const;
  virtual Tool::Base *baseFactory(Tool::Category) const;
};

} // namespace

#endif
