/***************************************************************************
 *   Copyright (C) 2006 Nicolas Hadacek <hadacek@kde.org>                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "jalv2_compile.h"

#include <qregexp.h>

#include "jalv2.h"
#include "common/common/misc.h"

//-----------------------------------------------------------------------------
void JALV2::CompileFile::logStderrLine(const QString &line)
{
//  if ( line.contains('\r') ) return; // what are those lines ?
  QRegExp rexp("([^:]*):([0-9]+):(.+)");
  if ( rexp.exactMatch(line) ) {
    PURL::Url url(rexp.cap(1));
    if ( !url.isInto(directory()) ) url = PURL::Url(directory(), rexp.cap(1));
    emit log(Log::Error, url.filename() + ":" + rexp.cap(2) + ": "
             + i18n("Error") + ": " + rexp.cap(3).stripWhiteSpace(),
               url, rexp.cap(2).toUInt()-1);
    return;
  }
  emit log(Log::Data, line); // unrecognized
}

//-----------------------------------------------------------------------------
QStringList JALV2::CompileFileStandalone::genericArguments(const Compile::Config &) const
{
  return "%I";
}

Compile::FileData::List JALV2::CompileFileStandalone::files() const
{
  Compile::FileData::List files;
  files += fileData(PURL::AsmGPAsm, Compile::Generated | Compile::InProject);
  files += fileData(PURL::Hex, Compile::Generated | Compile::Show | Compile::InProject);
  return files;
}

//-----------------------------------------------------------------------------
QStringList JALV2::CompileFileProject::genericArguments(const Compile::Config &) const
{
  return "%I";
}

Compile::FileData::List JALV2::CompileFileProject::files() const
{
  Compile::FileData::List files;
  files += fileData(PURL::AsmGPAsm, Compile::Generated | Compile::InProject);
  files += fileData(PURL::Hex, Compile::Generated | Compile::InProject);
  return files;
}
